
import collections
import os
from pequod_agent.utilities.system import execute

# start with a higher number, so the default loop0-7 devices are reserved for the system
FIRST_LOOP_DEVICE_NUMBER = 10


class LoopDevice(collections.namedtuple('LoopDevice', 'device file')):

    def associate(self, file: str):
        """
        Associate/setup the loop device with a file
        """
        execute('losetup {dev} {file}', dev=self.device, file=file)

    def detach(self):
        """
        Detach the loop device from its associated file
        """
        execute('losetup -d {dev}', dev=self.device)


def get_loop_devices():
    """
    Return all currently used/associated loop devices
    """
    for line in execute('losetup -a', False).splitlines():
        parts = line.strip().split(None)
        yield LoopDevice._make((parts[0].strip(':'), parts[2].strip('()')))


def make_loop_device() -> LoopDevice:
    """
    Allocate a new unused /dev/loopXX device and return the LoopDevice instance.
    This will create a new device if necessary.
    """
    num = FIRST_LOOP_DEVICE_NUMBER
    devices = set([dev.device for dev in get_loop_devices()])
    while True:
        num += 1
        loop_device = '/dev/loop{}'.format(num)
        if loop_device not in devices:
            break
    if not os.path.exists(loop_device):
        execute('mknod -m 660 {dev} b 7 {num}', dev=loop_device, num=num)
    return LoopDevice(device=loop_device, file=None)


if __name__ == '__main__':
    for loop in get_loop_devices():
        print(loop)