#!/usr/bin/env python3

import netaddr
import netifaces
from itertools import chain
from pequod_agent.utilities.system import execute
from yaml import safe_load


class NetworkCard:
    def __init__(self, device: str):
        self.device = device
        self.settings = self._get_settings()

    def _get_settings(self):
        # if we replace tabs with spaces ethtool output is valid yaml, hacky but it seems to work
        raw_info = execute('ethtool {dev}', _show_output=False, dev=self.device).replace('\t', '    ')
        info = safe_load(raw_info)
        settings = info['Settings for {dev}'.format(dev=self.device)]
        return settings

    @property
    def speeds_mbits(self) -> int:
        """
        Returns the speed in MBit/s
        """
        speed = int(self.settings['Speed'].replace('Mb/s', ''))
        return speed

    @classmethod
    def get_first_network_card(cls):
        for device in netifaces.interfaces():
            addresses = list(chain.from_iterable(addresses
                             for family, addresses
                             in netifaces.ifaddresses(device).items()
                             if family in [netifaces.AF_INET, netifaces.AF_INET6]))
            ips = (netaddr.IPAddress(address['addr']) for address in addresses if '%' not in address['addr'])
            is_loopback = any(ip.is_loopback() for ip in ips)
            if not is_loopback and not device.startswith('pequod'):
                return cls(device)